<?php
namespace KnitPay\Gateways\Paytm;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Pronamic\WordPress\Pay\Payments\Payment;
use Pronamic\WordPress\Pay\Payments\PaymentStatus;
use Exception;
use WP_Error;
use paytm\paytmchecksum\PaytmChecksum;

/**
 * Title: Paytm Gateway
 * Copyright: 2020-2022 Knit Pay
 *
 * @author Knit Pay
 * @version 4.9.0
 * @since 4.9.0
 */
class Gateway extends Core_Gateway {


	const NAME = 'paytm';

	/**
	 * Constructs and initializes an Paytm gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTML_FORM );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		$this->api = new API( $config->merchant_key, self::MODE_TEST === $config->mode );
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Core_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::PAYTM,
			PaymentMethods::UPI,
		);
	}

	/**
	 * Get available payment methods.
	 *
	 * @return array<int, string>
	 * @see Core_Gateway::get_available_payment_methods()
	 */
	public function get_available_payment_methods() {
		return $this->get_supported_payment_methods();
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$payment_currency = $payment->get_total_amount()
			->get_currency()
			->get_alphabetic_code();
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'Paytm only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'paytm_error', $currency_error );
			return;
		}

		$payment->set_transaction_id( $payment->key . '_' . $payment->get_id() );

		$payment_data = $this->get_payment_data( $payment );

		try {
			$transaction_token = $this->api->initiate_transaction( $payment_data );
			$payment->set_meta( 'transaction_token', $transaction_token );
		} catch ( Exception $e ) {
			$this->error = new WP_Error( 'paytm_error', $e->getMessage() );
			return;
		}

		$merchant_id = $payment_data['body']['mid'];
		$order_id    = $payment_data['body']['orderId'];

		$action_url = $this->api->get_endpoint() . 'theia/api/v1/showPaymentPage';
		$action_url = add_query_arg(
			array(
				'mid'     => $merchant_id,
				'orderId' => $order_id,
			),
			$action_url
		);
		$payment->set_action_url( $action_url );
	}

	/**
	 * Get Payment Data.
	 *
	 * @param Payment $payment
	 *            Payment.
	 *
	 * @return array
	 */
	private function get_payment_data( Payment $payment ) {
		$merchant_id  = $this->config->merchant_id;
		$merchant_key = $this->config->merchant_key;
		$website      = $this->config->website;

		$customer        = $payment->get_customer();
		$billing_address = $payment->get_billing_address();

		$cust_id = $customer->get_email();
		if ( empty( $cust_id ) ) {
			$cust_id = 'CUST_' . $payment->get_id();
		}

		// Create an array having all required parameters for creating checksum.
		// @see https://developer.paytm.com/docs/api/initiate-transaction-api/
		$paytmParams = array();

		$paytmParams['body'] = array(
			'requestType' => 'Payment',
			'mid'         => $merchant_id,
			'websiteName' => $website,
			'orderId'     => $this->get_order_id( $payment ),
			'callbackUrl' => $payment->get_return_url(),
			'txnAmount'   => array(
				'value'    => $payment->get_total_amount()->number_format( null, '.', '' ),
				'currency' => 'INR',
			),
			'userInfo'    => array(
				'custId' => $cust_id,
				'email'  => $customer->get_email(),
			),
		);

		if ( isset( $billing_address ) && ! empty( $billing_address->get_phone() ) ) {
			$paytmParams['body']['userInfo']['mobile'] = $billing_address->get_phone();
		}

		if ( null !== $customer->get_name() ) {
			$paytmParams['body']['userInfo']['firstName'] = $customer->get_name()->get_first_name();
			$paytmParams['body']['userInfo']['lastName']  = $customer->get_name()->get_last_name();
		}

		if ( ! empty( $payment->get_payment_method() ) && PaymentMethods::UPI === $payment->get_payment_method() ) {
			$paytmParams['body']['enablePaymentMode'] = array(
				array(
					'mode' => 'UPI',
				),
			);
		}

		// Generate checksum by parameters we have in body.
		$checksum = PaytmChecksum::generateSignature( wp_json_encode( $paytmParams['body'] ), $merchant_key );

		$paytmParams['head'] = array(
			'signature' => $checksum,
		);

		return $paytmParams;
	}

	/**
	 * Get output inputs.
	 *
	 * @see Core_Gateway::get_output_fields()
	 *
	 * @param Payment $payment
	 *            Payment.
	 *
	 * @return array
	 */
	public function get_output_fields( Payment $payment ) {
		return array(
			'mid'      => $this->config->merchant_id,
			'orderId'  => $this->get_order_id( $payment ),
			'txnToken' => $payment->get_meta( 'transaction_token' ),
		);
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
		if ( PaymentStatus::SUCCESS === $payment->get_status() ) {
			return;
		}

		$merchant_id  = $this->config->merchant_id;
		$merchant_key = $this->config->merchant_key;

		// Create an array having all required parameters for status query.
		$requestParamList = array(
			'body' => array(
				'mid'     => $merchant_id,
				'orderId' => $this->get_order_id( $payment ),
			),
		);

		$requestParamList['head']['signature'] = PaytmChecksum::generateSignature( wp_json_encode( $requestParamList['body'] ), $merchant_key );

		try {
			$transaction_status = $this->api->get_transaction_status( $requestParamList );
		} catch ( Exception $e ) {
			$this->error = new WP_Error( 'paytm_error', $e->getMessage() );
			return;
		}

		if ( $transaction_status->orderId !== $this->get_order_id( $payment ) ) {
			$this->error = new WP_Error( 'paytm_error', 'Something went wrong:<br><pre>' . print_r( $transaction_status, true ) . '</pre>' );
			return;
		}

		$note = '<strong>Paytm Parameters:</strong>';
		if ( isset( $transaction_status->txnId ) ) {
			$note .= '<br>Paytm txnId: ' . $transaction_status->txnId;
		}
		if ( isset( $transaction_status->bankTxnId ) ) {
			$note .= '<br>Paytm bankTxnId: ' . $transaction_status->bankTxnId;
		}
		$note .= '<br>Paytm orderId: ' . $transaction_status->orderId;
		$note .= '<br>Status: ' . $transaction_status->resultInfo->resultStatus;
		$note .= '<br>Message: ' . $transaction_status->resultInfo->resultMsg;

		$payment->add_note( $note );
		$payment->set_status( Statuses::transform( $transaction_status->resultInfo->resultStatus ) );

		if ( PaymentStatus::SUCCESS === $payment->get_status() ) {
			$payment->set_transaction_id( $transaction_status->txnId );
		}

		if ( PaymentStatus::OPEN === $payment->get_status() ) {
			$this->expire_old_payment( $payment );
		}
	}

	private function get_order_id( Payment $payment ) {
		$payment_description = $payment->get_description();
		$payment_description = str_replace( ' ', '_', $payment_description ); // Change space to underscore.
		$payment_description = preg_replace( '/[^a-zA-Z0-9_-]/s', '', $payment_description ); // Remove special characters.
		
		$order_id = $payment->key . '_' . $payment_description;
		return substr( $order_id, -50 );
	}

	private function expire_old_payment( $payment ) {
		// Make payment status as expired for payment older than 1 day.
		if ( DAY_IN_SECONDS < time() - $payment->get_date()->getTimestamp() && $this->config->expire_old_payments ) {
			$payment->set_status( PaymentStatus::EXPIRED );
		}
	}
}
